# Generated by roxygen2: do not edit by hand

S3method(plot,fahb_analysis)
S3method(plot,fahb_design)
S3method(print,fahb_analysis)
S3method(print,fahb_design)
export(check_priors)
export(fahb_analysis)
export(fahb_design)
export(fahb_problem)
export(forecast)
importFrom(rlang,.data)
