#' fahb: Feasibility Assessment with Hierarchical Bayes
#'
#' @description
#' Tools for simulation, design, and analysis of internal pilot trials
#' using frequentist and Bayesian operating characteristics.
#'
#' @importFrom rlang .data
#'
#' @keywords internal
"_PACKAGE"
