% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_problem.R
\name{fahb_problem}
\alias{fahb_problem}
\title{Build a \code{fahb_problem} object}
\usage{
fahb_problem(
  N = 320,
  m = 20,
  t = 0.5,
  n_ext = NULL,
  m_ext = NULL,
  rel_thr = 1.2,
  so_hps = c(30, 2.85),
  mean_rr_hps = c(2, 0.329),
  sd_rr_hps = c(30, 100)
)
}
\arguments{
\item{N}{target sample size.}

\item{m}{number of recruiting sites.}

\item{t}{timing of the pilot analysis in years.}

\item{n_ext}{number of participants to recruit to an external pilot.}

\item{m_ext}{number of sites to open in an external pilot.}

\item{rel_thr}{threshold which discriminates feasible and infeasible trials,
as a multiple of the expected time to recruit.}

\item{so_hps}{site opening rate hyperparameters (shape and rate for a Gamma prior).}

\item{mean_rr_hps}{mean site recruitment rate hyperparameters (mean and sd
for a lognormal prior).}

\item{sd_rr_hps}{variance in site recruitment rates hyperparameters (shape
and rate for a Gamma prior).}
}
\value{
an object of class \code{fahb_problem}
}
\description{
Given a trial design and a set of model hyperparameters, build an object
of class \code{fahb_problem}.
}
\examples{
fahb_problem()

}
