% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_analysis.R
\name{plot.fahb_analysis}
\alias{plot.fahb_analysis}
\title{Plot posterior distributions from a \code{fahb} analysis}
\usage{
\method{plot}{fahb_analysis}(x, ...)
}
\arguments{
\item{x}{object of class \code{fahb_analysis} as produced by \verb{fahb_analysis().}}

\item{...}{further arguments passed to or from other methods.}
}
\value{
no return value, called for side effects.
}
\description{
Takes an object of class \code{fahb_analysis} and plots the posterior
distributions of the predicted time for the trial to recruit and of the three
model parameters.
}
