% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_design.R
\name{plot.fahb_design}
\alias{plot.fahb_design}
\title{Plot operating characteristics of fahb designs}
\usage{
\method{plot}{fahb_design}(x, ...)
}
\arguments{
\item{x}{object of class \code{fahb_design} as produced by \verb{fahb_design().}}

\item{...}{further arguments passed to or from other methods.}
}
\value{
no return value, called for side effects.
}
\description{
Takes an object of class \code{fahb_design} and plots the estimated operating
characteristics of decision rules - based on standard progression criteria,
an approximate Bayesian analysis, or both.
}
