% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_priors.R
\name{check_priors}
\alias{check_priors}
\title{Plot prior distributions}
\usage{
check_priors(problem)
}
\arguments{
\item{problem}{an object of class \code{fahb_problem}.}
}
\value{
a list of \code{ggplot2} plots.
}
\description{
Given the hyperparameters encoded in a \code{fahb_problem} object, return plots of
the three prior distributions and a plot of the prior predictive distribution
of the recruitment rate at a randomly selected site.
}
\examples{
problem <- fahb_problem()
check_priors(problem)

}
