% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forecast.R
\name{forecast}
\alias{forecast}
\title{Generate probabilistic forecasts of trial recruitment}
\usage{
forecast(problem, n_sims = 10^4, overwrite = FALSE)
}
\arguments{
\item{problem}{an object of class \code{fahb_problem}.}

\item{n_sims}{number of replicates to use in the simulation.}

\item{overwrite}{boolean indicating if we want to overwrite any simulation data
currently held (defaults to FALSE).}
}
\value{
an object of class \code{fahb_problem}.
}
\description{
Generate probabilistic forecasts of trial recruitment
}
\examples{
problem <- fahb_problem()
problem <- forecast(problem, n_sims = 10^3)

}
