% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_analysis.R
\name{print.fahb_analysis}
\alias{print.fahb_analysis}
\title{Print a \code{fahb} analysis object}
\usage{
\method{print}{fahb_analysis}(x, ...)
}
\arguments{
\item{x}{object of class \code{fahb_analysis} as produced by \code{fahb_analysis()}.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
no return value, called for side effects.
}
\description{
The default print method for a \code{fahb_analysis} object.
}
