% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_design.R
\name{fahb_design}
\alias{fahb_design}
\title{Build a \code{fahb} design object}
\usage{
fahb_design(problem, quietly = TRUE)
}
\arguments{
\item{problem}{an object of class \code{fahb_problem}.}

\item{quietly}{if this argument is set to FASLE then information about which
steps have been completed will be printed to the console. Defaults to TRUE.}
}
\value{
an object of class \code{fahb_design}.
}
\description{
Given a \code{fahb_problem} object, find efficient progression decision rules.
These can include rules of the standard "progression criteria form", or rules
based on a Bayesian analysis of the pilot trial data, or both.
}
\examples{
problem <- forecast(fahb_problem(), n_sims = 500)
fahb_design(problem)

}
