% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb-package.R
\docType{package}
\name{fahb-package}
\alias{fahb}
\alias{fahb-package}
\title{fahb: Feasibility Assessment with Hierarchical Bayes}
\description{
Tools for simulation, design, and analysis of internal pilot trials
using frequentist and Bayesian operating characteristics.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://dtwilson.github.io/fahb/}
  \item \url{https://github.com/DTWilson/fahb}
  \item Report bugs at \url{https://github.com/DTWilson/fahb/issues}
}

}
\author{
\strong{Maintainer}: Duncan Wilson \email{d.t.wilson@leeds.ac.uk} (\href{https://orcid.org/0000-0001-7949-8718}{ORCID}) [copyright holder]

}
\keyword{internal}
